/**
* DO NOT EDIT THIS FILE.
* See the following change record for more information,
* https://www.drupal.org/node/2815083
* @preserve
**/

((Drupal, once) => {
  class ChoicesAutocomplete {
    constructor(element, labels) {
      this.element = element;
      this.config = this.element.dataset;
      this.url = this.config.autocompletePath;
      this.cache = {};
      this.value = null;
      const config = JSON.parse(element.dataset.choices);
      config.loadingText = Drupal.theme("ajaxProgressThrobber", config.loadingText);
      this.choices = new Choices(element, { ...config,
        shouldSort: false,
        allowHTML: true,
        removeItemButton: true,
        classNames: {
          containerOuter: "choices choices--autocomplete",
          item: "choices__item choices__item--autocomplete"
        }
      });
      element.addEventListener("change", this.clear.bind(this));

      if (labels) {
        const {
          choices
        } = this.choices.config;
        Object.keys(labels).forEach(value => {
          choices.forEach(choice => {
            if (choice.value === value) {
              choice.label = labels[value];
              choice.selected = true;
            }
          });
        });
        this.choices.removeActiveItems();
        this.choices.setChoices(choices, "value", "label", true);
      }

      element.addEventListener("showDropdown", () => {
        if (this.isMax()) {
          this.setNotice(this.choices.config.maxItemText);
        }
      });
      element.addEventListener("addItem", () => {
        if (this.isMax()) {
          this.choices.hideDropdown();
        }
      });
      this.choices.input.element.addEventListener("keydown", event => {
        if (!event.metaKey && (this.config.maxlength && event.key.length === 1 && event.target.value.length >= this.config.maxlength || this.config.pattern && event.key.length === 1 && !event.key.match(new RegExp(this.config.pattern, "g")))) {
          event.preventDefault();
        }
      });
      let timeout = null;
      element.addEventListener("search", event => {
        if (this.value !== event.detail.value) {
          this.clear();
          this.setNotice(this.choices.config.loadingText);
        }

        if (timeout) {
          clearTimeout(timeout);
        }

        timeout = setTimeout(this.search.bind(this, event), 500);
      });
    }

    setNotice(message) {
      this.choices.choiceList.clear();
      this.choices.choiceList.append(this.choices._getTemplate("notice", message));
    }

    isMax() {
      return this.choices.config.maxItemCount > 1 && this.choices.getValue(true).length >= this.choices.config.maxItemCount;
    }

    setState(loading) {
      if (loading) {
        this.choices.containerOuter.addLoadingState();
      } else {
        this.choices.containerOuter.removeLoadingState();
      }
    }

    search(event) {
      this.value = event.detail.value;

      if (this.cache[event.detail.value]) {
        this.populate([...this.cache[event.detail.value]], event.detail.value);
      } else {
        this.setState(true);
        fetch(`${this.url}?q="${encodeURIComponent(event.detail.value)}"`, {
          headers: {
            "Content-Type": "application/json; charset=utf-8"
          }
        }).then(response => response.json()).then(response => {
          this.cache[event.detail.value] = response;

          if (this.value === event.detail.value) {
            this.populate([...response], event.detail.value);
          }
        }).catch(() => {
          alert(Drupal.t("An error occurred. Please try again."));
        });
      }
    }

    populate(items, input) {
      this.setState(false);
      let values = this.choices.getValue(true);

      if (!Array.isArray(values)) {
        values = typeof values !== "undefined" ? [values] : [];
      }

      const ids = [];
      const labels = values.map(value => {
        const matches = value.match(/\s\(\d+\)"?$/);

        if (matches) {
          ids.push(matches[0].replace(/\D/g, "").trim());
        }

        return value.replace(/\s\(\d+\)"?$/, "").toLowerCase().trim();
      });
      items = items.filter(item => {
        labels.push(item.value.replace(/\s\(\d+\)"?$/, "").toLowerCase().trim());
        const matches = item.value.match(/\s\(\d+\)"?$/);

        if (matches) {
          return ids.indexOf(matches[0].replace(/\W/g, "").trim()) === -1;
        }

        return labels.indexOf(item.value.toLowerCase().trim()) === -1;
      });

      if (this.config.autocreate && (!this.config.minlength || input.length >= this.config.minlength) && labels.indexOf(input.toLowerCase().trim()) === -1) {
        items.unshift({
          value: input.trim(),
          label: input.trim()
        });
      }

      this.choices.setChoices(items, "value", "label", true);
    }

    clear() {
      this.choices.clearChoices();
    }

  }

  Drupal.behaviors.choicesAutocomplete = {
    attach: function attach(context, settings) {
      once("choices-autocomplete", "select.js-choices-autocomplete", context).forEach(element => {
        element.choicesAutocomplete = new ChoicesAutocomplete(element, settings.choices_autocomplete);
      });
    }
  };
})(Drupal, once);